@@ -8,28 +8,20 @@ from json import dumps
 class DataSet(object):
     def __init__(self, **kwargs):
         self.total_records = 0
+        self.not_commited = 0
 
         if 'backend' not in kwargs:
             raise KrakenousException('No backend specified')
         else:
             self.backend_name = kwargs['backend']
-
-        if self.backend_name == 'shelve':
-            if 'db_path' not in kwargs:
-                raise KrakenousException('No path to shelve file specified')
-            else:
-                self.db_data = {'db_path': kwargs['db_path']}
-                self.backend = __import__('.backend_shelve', fromlist=['backend_shelve'])
-                if os.path.isfile(self.db_data['db_path'] + '.db'):
-                    self.total_records = self.backend.data_records_amount(self.db_data)
-        elif self.backend_name == 'sqlite':
+        if self.backend_name == 'sqlite':
             if 'db_path' not in kwargs:
                 raise KrakenousException('No path to SQLite file specified')
             elif 'table_name' not in kwargs:
                 raise KrakenousException('No SQLite table name specified')
             else:
                 self.db_data = {'db_path': kwargs['db_path'], 'table_name': kwargs['table_name']}
-                self.backend = __import__('.backend_sqlite', fromlist=['backend_sqlite'])
+                self.backend = __import__('krakenous.backend_sqlite', fromlist=['backend_sqlite'])
                 if os.path.isfile(self.db_data['db_path']):
                     self.total_records = self.backend.data_records_amount(self.db_data)
                 else:
@@ -56,20 +48,15 @@ class DataSet(object):
         tentacle is a generator, returns a dictionary of stuff
         """
         records_added = 0
-        if self.backend_name in ('shelve', 'sqlite',):
-            if writeback != 0:
-                db = self.backend.open_db(self.db_data, True)
-            else:
-                db = self.backend.open_db(self.db_data, False)
+        db = self.backend.open_db(self.db_data)
         for tentacle_dict in tentacle(*args, **kwargs):
-            if self.backend_name == 'sqlite':
-                try:
-                    all_names = self.feature_names()
-                except KrakenousException:
-                    all_names = []
-                for tentacle_key in tentacle_dict:
-                    if tentacle_key not in all_names:
-                        self.backend.create_new_column(db, tentacle_key)
+            try:
+                all_names = self.feature_names()
+            except KrakenousException:
+                all_names = []
+            for tentacle_key in tentacle_dict:
+                if tentacle_key not in all_names:
+                    self.backend.create_new_column(db, tentacle_key)
             self._append_data_record_open(tentacle_dict, db)
             records_added += 1
             if writeback != 0 and self.total_records % writeback == 0:
@@ -79,36 +66,29 @@ class DataSet(object):
 
     def append_data_record(self, data_record: dict, serializers=None):
         """
-        do not write id field! (if exists in dict) (doesn't concern shelve)
+        do not write id field! (if exists in dict)
         """
-        if self.backend_name in ('shelve', 'sqlite',):
-            db = self.backend.open_db(self.db_data)
-        if self.backend_name == 'shelve':
-            for data_entry in data_record:
-                self.backend.write_data(db, self.total_records + 1, data_entry, data_record[data_entry])
-        elif self.backend_name == 'sqlite':
-            if 'id' in data_record.keys():
-                raise KrakenousException('Cannot write to id column')
-            try:
-                all_names = self.feature_names()
-            except KrakenousException:
-                all_names = []
-            for data_key in data_record:
-                if data_key not in all_names:
-                    self.backend.create_new_column(db, data_key)
-            self.backend.append_data_record(db, 1, data_record, serializers)
+        db = self.backend.open_db(self.db_data)
+        if 'id' in data_record.keys():
+            raise KrakenousException('Cannot write to id column')
+        try:
+            all_names = self.feature_names()
+        except KrakenousException:
+            all_names = []
+        for data_key in data_record:
+            if data_key not in all_names:
+                self.backend.create_new_column(db, data_key)
+        self.backend.append_data_record(db, 1, data_record, serializers)
         self.total_records += 1
         self.backend.close_db(db)
 
     def _append_data_record_open(self, data_record: dict, db):
         """
-        do not write id field! (if exists in dict) (doesn't concern shelve)
+        do not write id field! (if exists in dict)
         """
         self.total_records += 1
 
         self.backend.append_data_record(db, self.total_records, data_record)
-        # for data_entry in data_record:
-        #     backend.write_data(db, self.total_records, data_entry, data_record[data_entry])
 
     def insert_single(self, record_id: int, feature_name: str, data, overwrite_existing: bool=False, serializer=None):
         """
@@ -117,17 +97,28 @@ class DataSet(object):
         if not overwrite_existing and self.feature_exists(record_id, feature_name):
             raise KrakenousException('Feature with name "' + feature_name + '" already exists')
         db = self.backend.open_db(self.db_data)
-        if self.backend_name == 'sqlite':
-            if not serializer:
-                serializer = dumps
-            if feature_name not in self.feature_names():
-                self.backend.create_new_column(db, feature_name)
-        if self.backend_name == 'sqlite' and feature_name not in self.feature_names():
+        if not serializer:
+            serializer = dumps
+        if feature_name not in self.feature_names():
+            self.backend.create_new_column(db, feature_name)
+        self.backend.write_data(db, record_id, feature_name, data, serializer)
+        self.backend.close_db(db)
+
+    def insert_for_range(self, start_id: int, end_id: int, feature_name: str, data,
+                         overwrite_existing: bool=False, serializer=None):
+        """
+        insert a single value (data) for a range of ids
+        """
+        db = self.backend.open_db(self.db_data)
+        if not serializer:
+            serializer = dumps
+        if feature_name not in self.feature_names():
             self.backend.create_new_column(db, feature_name)
-        if self.backend_name == 'shelve':
-            self.backend.write_data(db, record_id, feature_name, data)
         else:
-            self.backend.write_data(db, record_id, feature_name, data, serializer)
+            if not overwrite_existing:
+                raise KrakenousException('Feature with name "' + feature_name + '" already exists')
+        for i, record_id in enumerate(range(start_id, end_id + 1)):
+            self.backend.write_data(db, record_id, feature_name, data[i], serializer)
         self.backend.close_db(db)
 
     def extract_feature_full_for_range(self, start_id: int, end_id: int,
@@ -141,16 +132,12 @@ class DataSet(object):
             raise KrakenousException('Feature with name "' + extractor_name + '" already exists')
         if extractor_name == 'id':
             raise KrakenousException('Cannot write to field "id"')
-        if writeback != 0:
-            db = self.backend.open_db(self.db_data, True)
-        else:
-            db = self.backend.open_db(self.db_data, False)
+        db = self.backend.open_db(self.db_data)
 
-        if self.backend_name == 'sqlite':
-            if not global_existence:
-                self.backend.create_new_column(db, extractor_name)
-            if not serializer:
-                serializer = dumps
+        if not global_existence:
+            self.backend.create_new_column(db, extractor_name)
+        if not serializer:
+            serializer = dumps
 
         for record_id in range(start_id, end_id + 1):
             additional_data = {}
@@ -159,12 +146,8 @@ class DataSet(object):
                     additional_data[column_name] = self.backend.read_single_data(db, record_id, column_name)
                 else:
                     additional_data[column_name] = record_id
-            if self.backend_name == 'shelve':
-                self.backend.write_data(db, record_id, extractor_name, extractor(additional_data,
-                                                                                 *args, **kwargs))
-            elif self.backend_name == 'sqlite':
-                self.backend.write_data(db, record_id, extractor_name, extractor(additional_data,
-                                                                                 *args, **kwargs), serializer)
+            self.backend.write_data(db, record_id, extractor_name, extractor(additional_data,
+                                                                             *args, **kwargs), serializer)
 
             if verbose != 0 and record_id % verbose == 0:
                 print(record_id)
@@ -206,56 +189,29 @@ class DataSet(object):
         if self.total_records == 0:
             raise KrakenousException('The dataset is empty!')
         else:
-            if writeback != 0:
-                db = self.backend.open_db(self.db_data, True)
-            else:
-                db = self.backend.open_db(self.db_data, False)
+            db = self.backend.open_db(self.db_data)
             for record_id in range(self.total_records):
                 self.backend.delete_data(db, record_id + 1, feature_name)
                 if writeback != 0 and record_id % writeback == 0:
                     self.backend.commit_db(db)
             self.backend.close_db(db)
 
-    def rename_feature(self, original_feature_name: str, new_feature_name: str, overwrite_existing: bool=False,
-                       writeback: int=0):
+    def rename_feature(self, original_feature_name: str, new_feature_name: str, overwrite_existing: bool=False):
         if self.backend_name == 'sqlite':
             raise KrakenousUnsupportedOperation('Renaming of features is not available for the SQLite backend')
-        if self.total_records == 0:
-            raise KrakenousException('The dataset is empty!')
-        if not overwrite_existing and self.feature_exists_global(new_feature_name):
-            raise KrakenousException('Feature with name "' + new_feature_name + '" already exists')
-
-        if self.backend_name == 'shelve':
-            if writeback != 0:
-                db = self.backend.open_db(self.db_data, True)
-            else:
-                db = self.backend.open_db(self.db_data, False)
-            for record_id in range(self.total_records):
-                self.backend.write_data(db, record_id + 1, new_feature_name, self.backend.read_single_data(db, record_id + 1,
-                                                                                          original_feature_name))
-                self.backend.delete_data(db, record_id + 1, original_feature_name)
-                if writeback != 0 and record_id % writeback == 0:
-                    self.backend.commit_db(db)
-            self.backend.close_db(db)
 
     def delete_records(self, record_ids: tuple, writeback: int=0):
-        if self.backend_name == 'shelve':
-            if writeback != 0:
-                db = self.backend.open_db(self.db_data, True)
-            else:
-                db = self.backend.open_db(self.db_data, False)
-        elif self.backend_name == 'sqlite':
-            raise KrakenousNotImplemented('Deleting records is not implemented for the SQLite backend')
-        if self.total_records == 0:
-            raise KrakenousException('The dataset is empty!')
-        for record_id in record_ids:
-            if record_id > self.total_records:
-                raise KrakenousException('Record with id=' + str(record_id) + ' does not exist!')
-            else:
-                self.backend.delete_record(db, record_id, self.total_records)
-                if writeback != 0 and record_id % writeback == 0:
-                    self.backend.commit_db(db)
-                self.total_records -= 1
+        raise KrakenousNotImplemented('Deleting records is not implemented for the SQLite backend')
+        # if self.total_records == 0:
+        #     raise KrakenousException('The dataset is empty!')
+        # for record_id in record_ids:
+        #     if record_id > self.total_records:
+        #         raise KrakenousException('Record with id=' + str(record_id) + ' does not exist!')
+        #     else:
+        #         self.backend.delete_record(db, record_id, self.total_records)
+        #         if writeback != 0 and record_id % writeback == 0:
+        #             self.backend.commit_db(db)
+        #         self.total_records -= 1
 
     def force_write_feature(self, feature_name: str, feature: list, writeback: int=0):
         if len(feature) != self.total_records:
@@ -264,10 +220,7 @@ class DataSet(object):
         elif feature_name == 'id':
             raise KrakenousException('Cannot write to field "id"')
         else:
-            if writeback != 0:
-                db = self.backend.open_db(self.db_data, True)
-            else:
-                db = self.backend.open_db(self.db_data, False)
+            db = self.backend.open_db(self.db_data)
             for record_id in range(self.total_records):
                 self.backend.write_data(db, record_id + 1, feature_name, feature[record_id])
                 if writeback != 0 and self.total_records % writeback == 0:
@@ -276,7 +229,7 @@ class DataSet(object):
 
     def feature_exists(self, record_id: int, feature_name: str) -> bool:
         db = self.backend.open_db(self.db_data)
-        if self.backend_name == 'sqlite' and feature_name not in self.feature_names():
+        if feature_name not in self.feature_names():
             self.backend.close_db(db)
             return False
         else:
@@ -300,12 +253,8 @@ class DataSet(object):
             raise KrakenousException('Feature "' + feature_name + '" does not exist')
         db = self.backend.open_db(self.db_data)
         result = []
-        if self.backend_name == 'sqlite':
-            for record_id in range(start_id, end_id + 1):
-                result.append(self.backend.read_single_data(db, record_id, feature_name, deserializer))
-        else:
-            for record_id in range(start_id, end_id + 1):
-                result.append(self.backend.read_single_data(db, record_id, feature_name))
+        for record_id in range(start_id, end_id + 1):
+            result.append(self.backend.read_single_data(db, record_id, feature_name, deserializer))
         self.backend.close_db(db)
         return result
 
@@ -321,35 +270,21 @@ class DataSet(object):
                 raise KrakenousException('Feature "' + feature_name + '" does not exist')
         db = self.backend.open_db(self.db_data)
         result = []
-        if self.backend_name == 'sqlite':
-            for record_id in range(start_id, end_id + 1):
-                result.append(self.backend.read_multiple_data(db, record_id, feature_names, deserializers))
-        else:
-            for record_id in range(start_id, end_id + 1):
-                result.append(self.backend.read_multiple_data(db, record_id, feature_names))
+        for record_id in range(start_id, end_id + 1):
+            result.append(self.backend.read_multiple_data(db, record_id, feature_names, deserializers))
         self.backend.close_db(db)
         return result
 
-    def feature_names(self, record_id=-1) -> list:
+    def feature_names(self) -> list:
         if self.total_records == 0:
             raise KrakenousException('The dataset is empty!')
         db = self.backend.open_db(self.db_data)
-        return_list = []
-        if self.backend_name == 'shelve':
-            if record_id == -1:
-                for r_id in range(self.total_records):
-                    tmp_list = self.backend.all_data_names(db, r_id + 1)
-                    for tmp_f in tmp_list:
-                        if tmp_f not in return_list:
-                            return_list.append(tmp_f)
-            else:
-                return_list = self.backend.all_data_names(db, record_id)
-        elif self.backend_name == 'sqlite':
-            return_list = self.backend.all_data_names(db)
+        return_list = self.backend.all_data_names(db)
         self.backend.close_db(db)
         return return_list
 
-    def _single_data_record_open(self, record_id, db, column_names: tuple=(), deserializers: dict=None) -> dict:
+    def _single_data_record_open(self, record_id, db, column_names: tuple=(), deserializers: dict=None,
+                                 filters=None) -> dict:
         """
         returns a dict
         """
@@ -360,11 +295,15 @@ class DataSet(object):
         else:
             for column_name in column_names:
                 if column_name != 'id':
-                    if self.backend_name == 'sqlite' and deserializers and column_name in deserializers:
+                    if deserializers and column_name in deserializers:
                         result_dict[column_name] = self.backend.read_single_data(db, record_id, column_name,
-                                                                                 deserializers[column_name])
+                                                                                 deserializers[column_name],
+                                                                                 filters)
                     else:
-                        result_dict[column_name] = self.backend.read_single_data(db, record_id, column_name)
+                        result_dict[column_name] = self.backend.read_single_data(db, record_id, column_name,
+                                                                                 filters=filters)
+                        if result_dict[column_name] is None:
+                            return None
                 else:
                     result_dict['id'] = record_id
         return result_dict
@@ -376,28 +315,21 @@ class DataSet(object):
         result_dict = {}
         db = self.backend.open_db(self.db_data)
         if column_names == ():
-            if self.backend_name == 'shelve':
-                result_dict = self.backend.read_all_data(db, record_id)
-            elif self.backend_name == 'sqlite':
-                tmp_res = self.backend.read_all_data(db, record_id, deserializers)
-                all_column_names = self.backend.all_data_names(db)
-                for column_name_enum in enumerate(all_column_names):
-                    result_dict[column_name_enum[1]] = tmp_res[column_name_enum[0]]
+            tmp_res = self.backend.read_all_data(db, record_id, deserializers)
+            all_column_names = self.backend.all_data_names(db)
+            for column_name_enum in enumerate(all_column_names):
+                result_dict[column_name_enum[1]] = tmp_res[column_name_enum[0]]
             result_dict['id'] = record_id
         else:
             if 'id' in column_names:
                 result_dict['id'] = record_id
-            if self.backend_name == 'shelve':
-                for column_name in column_names:
-                    result_dict[column_name] = self.backend.read_single_data(db, record_id, column_name)
-            elif self.backend_name == 'sqlite':
-                for column_name in column_names:
-                    if column_name != 'id':
-                        if deserializers and column_name in deserializers:
-                            result_dict[column_name] = self.backend.read_single_data(db, record_id, column_name,
-                                                                                     deserializers[column_name])
-                        else:
-                            result_dict[column_name] = self.backend.read_single_data(db, record_id, column_name)
+            for column_name in column_names:
+                if column_name != 'id':
+                    if deserializers and column_name in deserializers:
+                        result_dict[column_name] = self.backend.read_single_data(db, record_id, column_name,
+                                                                                 deserializers[column_name])
+                    else:
+                        result_dict[column_name] = self.backend.read_single_data(db, record_id, column_name)
         self.backend.close_db(db)
         return result_dict
 
@@ -409,14 +341,13 @@ class DataSet(object):
         db = self.backend.open_db(self.db_data)
         end_id = self.get_end_id(start_id, end_id)
         for record_id in range(start_id, end_id + 1):
-            if filters:
-                data_record = self._single_data_record_open(record_id, db, column_names, deserializers)
-                flag = True
-                for column_name in filters:
-                    if data_record[column_name] != filters[column_name]:
-                        flag = False
-                if flag:
-                    yield data_record
+            if filters and 'id' in filters:
+                if record_id == filters['id']:
+                    res = self._single_data_record_open(filters['id'], db, column_names, deserializers, filters)
+                    if res is not None:
+                        yield res
             else:
-                yield self._single_data_record_open(record_id, db, column_names, deserializers)
+                res = self._single_data_record_open(record_id, db, column_names, deserializers, filters)
+                if res is not None:
+                    yield res
         self.backend.close_db(db)
\ No newline at end of file
